#define  USE_HY17M28_2M
/******************************************************************************
 * -------------------------------------------------------------------------- *
 * Copyright 2021 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M28                                                           *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <GPIO.h>
#include <CLK.h>
#include <INT.h>
#include <PWR.h>
#include <RST.h>
#include <ADC.h>
/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
//#define ADC_ChopperMode

#ifdef USE_HY17M28_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M28_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M28_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M28_16M
#define HAO_16MHZ
#endif
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int del);
void ADCInit(void);
void CLOCKInit(void);
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
volatile typedef union _Flag
{
 unsigned int  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned Reserved:15;
  };
} Flag;

//signed long
//range 8388607~-8388608 (dec)
//range 0x007F FFFF ~ 0xFF80 0000 (hex)
signed long  ADC_ISRTemp;
signed long  ADC_ISRTemp_Buffer[8];
unsigned int counterA;
Flag   Flagbits;
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
  counterA=0;
  Flagbits.b_ADCdone=0;
  CLOCKInit();
  ADCInit();
  GIE_Enable();

  while(1)
  {
    if(Flagbits.b_ADCdone==1)
    {
      ADC_ISRTemp_Buffer[counterA]=ADC_ISRTemp;
      Flagbits.b_ADCdone=0;
      counterA++ ;
      if( counterA==8 )
      {
        counterA=0;
      }
    }
  }

}

/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt
{
  NOP();
  //ADC Event
  if(ADCIF_IsFlag())
  {
    ADCIF_ClearFlag();
    ADC_ISRTemp=ADCR; //ADC_GetData();
    Flagbits.b_ADCdone=1;
  }
}
/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int del)
{
  while(--del)
  NOP();
}
/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

}

/*----------------------------------------------------------------------------*/
/* ADC Init Function                                                          */
/*----------------------------------------------------------------------------*/
void ADCInit(void)
{
  //==========Setup Power=====================================
  PWR_VDDAOpen(LDOC_2V4);  //refer to the Datasheet for LDOC typical voltage

  //==========Setup ADC=================================
#if defined(HAO_2MHZ)
  //HAO=2MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV2,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif

#if defined(HAO_4MHZ)
  //HAO=4MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV4,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif

#if defined(HAO_8MHZ)
  //HAO=8MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV8,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif

#if defined(HAO_16MHZ)
  //HAO=15.667MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV16,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif

#if defined (ADC_ChopperMode)
  CSFON_Enable();
  ADC_ENINXCH_Enable();
  ADC_DAFM_CHOPPER();
  ADC_ENCH_Enable();
  ADC_Enable();
  ADC_CMFREnable();     //CMFR=1, Comb Filter Reset
#endif

}
/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
