#define  USE_HY17M28_2M
/******************************************************************************
 * -------------------------------------------------------------------------- *
 * Copyright 2021 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M28                                                           *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <INT.h>
#include <CLK.h>
#include <PWR.h>
#include <ADC.h>
#include <LED.h>
#include <TMR.h>
#include <RST.h>
#include "lcdTable.h"
/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
//#define ADC_ChopperMode

#ifdef USE_HY17M28_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M28_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M28_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M28_16M
#define HAO_16MHZ
#endif
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int del);
void CLOCKInit(void);
void LED_Display_Init(void);
void LED_Display_show(signed long DATA);
void LED_Display_h(unsigned char ledx, unsigned char On_OFF);
void ADCInit(void);
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
unsigned char LED[8];
unsigned char LEDScan;

volatile typedef union _Flag
{
 unsigned int  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned Reserved:15;
  };
} Flag;
Flag   Flagbits;

//signed long
//range 8388607~-8388608 (dec)
//range 0x007F FFFF ~ 0xFF80 0000 (hex)
signed long  ADC_ISRTemp;
signed long  ADCData1;
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
  CLOCKInit();
  LED_Display_Init();
  ADCInit();
  GIE_Enable();

  while(1)
  {
    if(Flagbits.b_ADCdone==1)
    {
      Flagbits.b_ADCdone=0;
      LED_Display_show(ADC_ISRTemp);
      Delay(10000);
    }
  }

}
/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt
{
    //TMA Event
    //A21023 V01 version
	if(TA1CIF_IsFlag())
	{
		TA1CIF_ClearFlag();
		TMA1R=0;
		LEDScan=LEDScan>>1;
		if(LEDScan==0)
		{LEDScan=0x80;}

		switch(LEDScan)
		{
			case 0x01:
			{
				PT2= 0;
				TRISC2=LED[0] | 0x01;
				PT2=~(LED[0] & 0xFE);
				break;
			}
			case 0x02:
			{
				PT2= 0;
				TRISC2=LED[1] | 0x02;
				PT2=~(LED[1]& 0xFD);
				break;
			}
			case 0x04:
			{
				PT2= 0;
				TRISC2=LED[2] | 0x04;
				PT2=~(LED[2] & 0xFB);
				break;
			}
			case 0x08:
			{
				PT2= 0;
				TRISC2=LED[3] | 0x08;
				PT2=~(LED[3]& 0xF7);
				break;
			}
			case 0x10:
			{
				PT2= 0;
				TRISC2=LED[4] | 0x10;
				PT2=~(LED[4]& 0xEF);
				break;
			}
			case 0x20:
			{
				PT2= 0;
				TRISC2=LED[5] | 0x20;
				PT2=~(LED[5]& 0xDF);
				break;
			}
			case 0x40:
			{
				PT2= 0;
				TRISC2=LED[6] | 0x40;
				PT2=~(LED[6]& 0xBF);
				break;
			}
			case 0x80:
			{
				PT2= 0;
				TRISC2=LED[7] | 0x80;
				PT2=~(LED[7]& 0x7F);
				break;
			}
		}
	}

  //ADC Event
  if(ADCIF_IsFlag())
  {
    ADCIF_ClearFlag();
    ADC_ISRTemp=ADCR; //ADC_GetData();
    Flagbits.b_ADCdone=1;
  }

}
/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

}
/*----------------------------------------------------------------------------*/
/* ADC Init Function                                                          */
/*----------------------------------------------------------------------------*/
void ADCInit(void)
{
  //==========Setup Power=====================================
  PWR_VDDAOpen(LDOC_2V4);  //refer to the Datasheet for LDOC typical voltage

  //==========Setup ADC=================================
#if defined(HAO_2MHZ)
  //HAO=2MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV2,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif

#if defined(HAO_4MHZ)
  //HAO=4MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV4,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif

#if defined(HAO_8MHZ)
  //HAO=8MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV8,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif

#if defined(HAO_16MHZ)
  //HAO=15.667MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV16,INP_AI0,INN_AI1,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_65536);
#endif

#if defined (ADC_ChopperMode)
  CSFON_Enable();
  ADC_ENINXCH_Enable();
  ADC_DAFM_CHOPPER();
  ADC_ENCH_Enable();
  ADC_Enable();
  ADC_CMFREnable();     //CMFR=1, Comb Filter Reset
#endif

}

/*----------------------------------------------------------------------------*/
/* LED Init Function                                                          */
/*----------------------------------------------------------------------------*/
void LED_Display_Init(void)
{
  LEDSetting(CCLevel_15mA);     //Set PT2 constant current 15mA	
  LEDScan= 0;

  TMA1_Open(TMAS_DMSCK,DTMA_TMACKDIV2,2);   //Enable TMA, set LED scan rate=2
  TA1IE_Disable();    //Disable TMA1 Interrupt
  TA1CIE_Enable();    //Enable TMA1C Interrupt

}

/*----------------------------------------------------------------------------*/
/* LED_Display_show Function                                                      */
/*----------------------------------------------------------------------------*/
void LED_Display_show(signed long DATA)
{
  //display range +8388607~-8388608 (dec)
  //display range 0x7FFFFF~ 0x800000 (hex)
  ADCData1=DATA;
  if((ADCData1<0)||(ADCData1>0x80000000))  // plus-minus sign judgment
  {
	  ADCData1=~ADCData1;
	  ADCData1++;
      LED[7]=LED[7] | 0x40;  //UartTxBuffer[0]='-';
  }
  else
  {
      if(LED[7]==0x40)
      {LED[7]=LED[7] - 0x40;}  //UartTxBuffer[0]='+';
  }
  DATA=ADCData1;

  ADCData1= seg[DATA%10];
  LED[6]= GetDisplay(ADCData1,7);
  DATA /=10;

  ADCData1= seg[DATA%10];
  LED[5]= GetDisplay(ADCData1,6);
  DATA /=10;

  ADCData1= seg[DATA%10];
  LED[4]= GetDisplay(ADCData1,5);
  DATA /=10;

  ADCData1= seg[DATA%10];
  LED[3]= GetDisplay(ADCData1,4);
  DATA /=10;

  ADCData1= seg[DATA%10];
  LED[2]= GetDisplay(ADCData1,3);
  DATA /=10;

  ADCData1= seg[DATA%10];
  LED[1]= GetDisplay(ADCData1,2);
  DATA /=10;

  ADCData1= seg[DATA%10];
  LED[0]= GetDisplay(ADCData1,1);

}

/*----------------------------------------------------------------------------*/
/* LED_Display_h Function                                                         */
/*----------------------------------------------------------------------------*/
void LED_Display_h(unsigned char ledx, unsigned char On_OFF)
{

  unsigned char LEDScan_h, On_OFF_h ;
  LEDScan_h=ledx;
  On_OFF_h=On_OFF;

  switch(LEDScan_h)
  {
    case 0x1:
    {
      if(On_OFF_h==1)
      {LED[7]=LED[7] | 0x01;}
      if(On_OFF_h==0)
      {LED[7]=LED[7] -0x01;}
      break;
    }
    case 0x2:
    {
      if(On_OFF_h==1)
      {LED[7]=LED[7] | 0x02;}
      if(On_OFF_h==0)
      {LED[7]=LED[7] - 0x02;}
      break;
    }
    case 0x3:
    {
      if(On_OFF_h==1)
      {LED[7]=LED[7] | 0x04;}
      if(On_OFF_h==0)
      {LED[7]=LED[7] - 0x04;}
      break;
    }
    case 0x4:
    {
      if(On_OFF_h==1)
      {LED[7]=LED[7] | 0x08;}
      if(On_OFF_h==0)
      {LED[7]=LED[7] - 0x08;}
      break;
    }
    case 0x5:
    {
      if(On_OFF_h==1)
      {LED[7]=LED[7] | 0x10;}
      if(On_OFF_h==0)
      {LED[7]=LED[7] - 0x10;}
      break;
    }
    case 0x6:
    {
      if(On_OFF_h==1)
      {LED[7]=LED[7] | 0x20;}
      if(On_OFF_h==0)
      {LED[7]=LED[7] - 0x20;}
      break;
    }
    case 0x7:
    {
      if(On_OFF_h==1)
      {LED[7]=LED[7] | 0x40;}
      if(On_OFF_h==0)
      {LED[7]=LED[7] - 0x40;}
      break;
    }
  }

}

/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int del)
{
  while(--del)
  NOP();
}
/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
