#define  USE_HY17M28_2M
/******************************************************************************
 * BIE_Demo                                                                   *
 * -------------------------------------------------------------------------- *
 * Copyright 2022 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M28                                                           *
 * Project Name : TMA                                                         *
 * Created Date : 2022/3/31 By Cyril                                          *
 *                                                                            *
 ******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <INT.h>
#include <BIE.h>
#include <CLK.h>
#include <RST.h>


/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#ifdef USE_HY17M28_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M28_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M28_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M28_16M
#define HAO_16MHZ
#endif

/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void);
void EERDWrite(void);
void EERDCLR(void);
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
    CLOCKInit();  //Operating BIE, CPU clock select HAO
    GIE_Disable();  //Disable the GIE interrupt brfore operating BIE
    
    BIE_EEPMode(EEMODE_64bytes);  //Select EEPROM mode
    EERDCLR();  //Clear EERD0~63
          
    EERDWrite();  //Write data to EERD
    BIE_DataWrite();  //Write EERD to BIE
    
    EERDCLR();  //Clear EERD0~63
    BIE_DataReload();  //Reload data to EERD
    NOP();
    
    GIE_Enable();
    while(1);

}

/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt
{

}

/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

}

/*----------------------------------------------------------------------------*/
/* EEPROM Data Write Function                                                 */
/*----------------------------------------------------------------------------*/
void EERDWrite(void)
{
  unsigned char eepcount=0;

  FSR0= &EERD0;
  for(eepcount=1;eepcount<=64;eepcount++)
  {
    POINC0= eepcount;
  }

}

/*----------------------------------------------------------------------------*/
/* EEPROM Data Clear Function                                                 */
/*----------------------------------------------------------------------------*/
void EERDCLR(void)
{
  unsigned char eepcount;

  FSR0= &EERD0;
  for(eepcount=0;eepcount<=63;eepcount++)
  {
    POINC0=0;
  }

}
/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
