#define  USE_HY17M28_2M
/******************************************************************************
 * I2C_E2PROM.c                                                        *
 * -------------------------------------------------------------------------- *
 * Copyright 2022 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 *        HY17M28               24C02A                                     *
 *---------------------         ------------------                                                                   *
 *                        |          |                                         *
 *                        |          |                                          *
 *               VDD |----->  | VDD                                                           *
 *             PT3.0 |----->  | SCL                                                 *
 *             PT3.1 |----->  | SDA                                               *
 *               VSS |----->  | VSS                                                    *
 *                       |           |                                         *
 *                       |           |                                          *
 *-------------------            ----------------                                                                *
 *                                                                                                         *
 * EEPROM Address                                                                                                       *
 * addr[0x00]=0xaa                                                                           *
 * addr[0x01]=0x02                                                                           *
 * addr[0x02]=0x03                                                                           *
 * addr[0x03]=0x04                                                                           *
 * addr[0x04]=0x05                                                                           *

 * For External Input                                                         *
 * IC Body: HY17M28                                                           *

 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <INT.h>
#include <CLK.h>
#include <PWR.h>
#include <I2C.h>
#include <GPIO.h>
#include "EEPROM_24Cxx.h"
/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#ifdef USE_HY17M28_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M28_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M28_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M28_16M
#define HAO_16MHZ
#endif

#define I2C_PORT GSCL_PT16

#define  I2CBufferSize  10
#define I2C_WRITE  0x00       // I2C WRITE command
#define I2C_READ   0x01       // I2C READ command

/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void Delayms(unsigned int ms);
void I2CInit(void);
void CLOCKInit(void);
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
unsigned char I2C_Read_Buffer[I2CBufferSize];
unsigned char I2C_RW;
unsigned char I2C_TARGET;      // Target I2C slave address
unsigned char I2C_Sendbuf[I2CBufferSize];
unsigned char I2C_Recbuf[I2CBufferSize];
volatile unsigned char I2C_EndFlag;
unsigned char EEPROM_WriteData[5] = {0x02,0x03,0x04,0x05,0x8A};
unsigned int I2C_DataTxLen,I2C_DataTxIndex,I2C_DataRxLen,I2C_DataRxIndex;
unsigned char i,temp;
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
	CLOCKInit();
	I2CInit();
	for(i=0;i<I2CBufferSize;i++)
	{
		I2C_Read_Buffer[i]=0x00;	//CLR I2C_Read_Buffer[]
	}

	GIE_Enable();

	// I2C single I2C_WRITE & I2C_READ
	EEPROM_ByteWrite(0x00,0xAA);               //Single Byte I2C_WRITE word address 0x00, and I2C_WRITE data 0x01
	Delayms(10);                               //Delay for EEPROM 24C02 I2C_WRITE Cycle Time
	I2C_Read_Buffer[0]=EEPROM_ByteRead(0x00);  //Single Byte I2C_READ word address 0x00, the I2C_READ value is 0x01

	// I2C sequential I2C_WRITE & I2C_READ
	EEPROM_WriteArray(0x01,EEPROM_WriteData,5); //I2C_WRITE array data to the word address from 0x01 to 0x04
	Delayms(10);                                //Delay for EEPROM 24C02 I2C_WRITE Cycle Time
	EEPROM_ReadArray(I2C_Read_Buffer, 0x00, 2); //sequential I2C_READ data from 0x00 to 0x01
	EEPROM_ReadArray(I2C_Read_Buffer, 0x00, 5); //sequential I2C_READ data from 0x00 to 0x04
	EEPROM_ReadArray(I2C_Read_Buffer, 0x00, 6); //sequential I2C_READ data from 0x00 to 0x05
	while(1);
}
/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt
{
	if(I2CIF_IsFlag() &&I2C_EndFlag==0)  //Get I2C Interrupt Flag
	{
    	switch(I2C_STAState())
    	{
	      case 0x90:  //MACTFlag+RWFlag
	                 //START has been transmitted
	            I2C_SendData(I2C_TARGET);  //Send Slave Address & R/W Bit
	            I2C_Ctrl(0,0,0,0);          //Clear all I2C flag
	            break;

	      case 0x84:  //MACTFlag+ACKFlag
	                 //Slave A + W has been transmitted. ACK has been received.
	            I2C_SendData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
	            I2C_Ctrl(0,0,0,0);      //Clear all I2C flag
	            break;

	      case 0x80: //MACTFlag
	                //Slave A + W has been transmitted. ACK has been received.
	      		I2C_SendData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
	            I2C_Ctrl(0,0,0,0);      //Clear all I2C flag
	            break;

	      case 0x30:

	            I2C_Ctrl(0,0,0,0);      //Clear all I2C flag
	            I2C_EndFlag=1;
	            break;

	      case 0x8C:  //MACTFlag+DFFlag+ACKFlag
	                //DATA has been transmitted and ACK has been received
	            if(I2C_DataTxIndex<I2C_DataTxLen)
	            {
	              I2C_SendData(I2C_Sendbuf[I2C_DataTxIndex++]);  //Send Data to Slave
	              I2C_Ctrl(0,0,0,0);    // Clear all I2C flag
	            }
	            else
	            {
		            if(I2C_RW == I2C_WRITE)
		            {
		            	I2C_Ctrl(0,1,0,0);  //I2C as master sends STOP signal
		            	I2C_EndFlag=1;
		            }
		            else if(I2C_RW == I2C_READ)
		            I2C_Ctrl(1,0,0,0);  //I2C as master sends START signal
		            I2C_DataTxIndex=0;
	            }
	            break;

	      case 0x88:  //MACTFlag+DFFlag
	                 //DATA has been transmitted and NACK has been received
	            I2C_Ctrl(0,1,0,0);    //I2C as master sends STOP signal
	            I2C_DataTxIndex=0;
	            I2C_EndFlag=1;
	            break;

	      case 0xB0:
	                 //A repeated START has been transmitted.
	            I2C_SendData(I2C_TARGET | I2C_READ);  //Send Slave Address & R/W Bit
	            I2C_Ctrl(0,0,0,0);    //Clear all I2C flag
	            break;

	      case 0x94:  //MACTFlag+RWFlag+ACKFlag
	                 //Slave A + R has been transmitted. ACK has been received.
	            if(I2C_DataRxLen>1)
	            {
	              I2C_Ctrl(0,0,0,1);  //Set ACK bit
	            }else{
	              I2C_Ctrl(0,0,0,0);  //Clear all I2C flag
	            }
	            break;

	      case 0x9C:  //MACTFlag+RWFlag+DFFlag+ACKFlag
	                //Data byte has been received. ACK has been transmitted.
	            I2C_ReceiveDATA(I2C_Recbuf[I2C_DataRxIndex++]);
	            if((I2C_DataRxLen-1)>I2C_DataRxIndex)
	            {
	              I2C_Ctrl(0,0,0,1);  //Set ACK bit
	            }
	            else
	            {
	              I2C_Ctrl(0,0,0,0);  //Clear all I2C flag
	            }
	            break;

	      case 0x98:  //MACTFlag+RWFlag+DFFlag
	                //Data byte has been received. NACK has been transmitted.
	            I2C_ReceiveDATA(I2C_Recbuf[I2C_DataRxIndex++]);
	            I2C_Ctrl(0,1,0,0);  //I2C as master sends STOP signal
	            I2C_EndFlag=1;
	            break;

	      default:
	            I2C_Ctrl(0,0,0,0);  //Clear all I2C flag
	            I2C_EndFlag=1;
	            break;
    	}
		I2C_I2CINT_CLEAR();		//CLR I2C INT Flag
		I2C_I2CER_CLEAR();		//CLR I2C error Flag
		I2CIF_ClearFlag();				//CLR I2CIF
  	}

  	if(I2CERIF_IsFlag())  //Get I2C Error Interrupt Flag
  	{
	    I2C_EndFlag=1;
		I2C_I2CINT_CLEAR();		//CLR I2C INT Flag
		I2C_I2CER_CLEAR();		//CLR I2C error Flag
		I2CERIF_ClearFlag();		//CLR I2CERIF
	    I2C_Ctrl(0,0,0,0);          //Clear all I2C flag
  	}

}
/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

}

/*----------------------------------------------------------------------------*/
/* I2C Init Function                                                        */
/*----------------------------------------------------------------------------*/
void I2CInit(void)
{
  //GPIO Init
  GPIO_GSCLSet(I2C_PORT);

#if defined(HAO_2MHZ)
  I2C_Open(40);  //Default CPU clock is 2MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 2000000/[4*(4+1)]=100kHz
#endif
#if defined(HAO_4MHZ)
  I2C_Open(9);  //Default CPU clock is 1MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 4000000/[4*(9+1)]=100kHz
#endif
#if defined(HAO_8MHZ)
  I2C_Open(19);  //Default CPU clock is 8MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 8000000/[4*(19+1)]=100kHz
#endif
#if defined(HAO_16MHZ)
  I2C_Open(39);  //Default CPU clock is 16MHz, Data Baud Rate : (I2CLK)/[4x(CRG+1)]= 16000000/[4*(39+1)]=100kHz
#endif
}

/*----------------------------------------------------------------------------*/
/* Delay Function                                                             */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
	for(;num>0;num--)
    __asm__("NOP");
}


void Delayms(unsigned int ms)
{
  unsigned char t;
  for(;ms>0;ms--)        //@HAO=1.843MHz
    for(t=114;t>0;t--)
    { __asm__("NOP"); }
}

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
