#define  USE_HY17M28_2M
/******************************************************************************
 * -------------------------------------------------------------------------- *
 * Copyright 2021 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M28                                                           *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <INT.h>
#include <PWR.h>
#include <CLK.h>
#include <GPIO.h>
#include <RST.h>

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#ifdef USE_HY17M28_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M28_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M28_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M28_16M
#define HAO_16MHZ
#endif
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int del);
void GPIOInit(void);
void CLOCKInit(void);
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
volatile typedef union _Flag
{
  unsigned int  _byte;
  struct
  {
    unsigned b_PT1INT0done:1;
    unsigned b_PT1INT1done:1;
    unsigned b_PT1INT2done:1;
    unsigned b_PT1INT3done:1;
    unsigned b_PT1INT4done:1;
    unsigned b_PT1INT5done:1;
    unsigned b_PT1INT6done:1;
    unsigned b_PT2INT0done:1;
    unsigned b_PT2INT1done:1;
    unsigned b_PT2INT2done:1;
    unsigned b_PT2INT3done:1;
    unsigned b_PT2INT4done:1;
    unsigned b_PT2INT5done:1;
    unsigned b_PT2INT6done:1;
    unsigned b_PT2INT7done:1;
    unsigned b_PT3INT0done:1;	//PT31~PT37 not design in this demo
  };
} Flag;
Flag   Flagbits;
unsigned int counterA;
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{

  counterA=0;
  CLOCKInit();
  GPIOInit();
  Flagbits.b_PT1INT0done=0;
  Flagbits.b_PT1INT1done=0;
  GIE_Enable();

  while(1)
  {

    if(Flagbits.b_PT1INT0done==1)  //PT1.0
    {
      Flagbits.b_PT1INT0done=0;
      CSFON_Enable();
      ENBOR2_Disable();
      PWR_BGRDisable();
      CLK_IDLE_HAOSet(IDLEM_HAOAuto);  //OSC switch to HAO ,when the GPIO interrupt wake up
      Idle();                      //IDLE mode
      counterA++;
    }
    if(Flagbits.b_PT1INT1done==1)  //PT1.1
    {
      Flagbits.b_PT1INT1done=0;
      CSFON_Enable();
      ENBOR2_Disable();
      PWR_BGRDisable();
      CLK_OSCSelect(OSCS_LPO);
      CLK_HAODisable();
      Sleep();                     //Sleep mode
      counterA--;
    }

  }

}
/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt
{
  NOP();
  //GPIO Event

  if(E0IF_IsFlag())
  {
    Flagbits.b_PT1INT0done=1;
    E0IF_ClearFlag();
  }

  if(E1IF_IsFlag())
  {
    Flagbits.b_PT1INT1done=1;
    E1IF_ClearFlag();
  }
}
/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int del)
{
  while(--del)
  NOP();
}
/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif
}

/*----------------------------------------------------------------------------*/
/* GPIO Init Function                                                         */
/*----------------------------------------------------------------------------*/
void GPIOInit(void)
{
  //GPIO initial on PT1.0 & PT1.1 (INPUT)
  GPIO_PT1DigitalEnable(PT10);  //PT1.0 digital input enable
  GPIO_PT1DigitalEnable(PT11);  //PT1.1 digital input enable
  GPIO_PT1SETPU(PT10);  //PT1.0 input high
  GPIO_PT1SETPU(PT11);  //PT1.1 input high
  INTEG0Sel(INTEG0_EDGEFALL);  //PT1.0. INTEG0[1:0]=00b, falling edge
  INTEG1Sel(INTEG1_EDGEFALL);  //PT1.0. INTEG0[1:0]=00b, falling edge
  E0IE_Enable();  //PT1.0 E0IE enable
  E1IE_Enable();  //PT1.1 E1IE enable
  E0IF_ClearFlag();  //PT1.0 E0IF=0b
  E1IF_ClearFlag();  //PT1.1 E1IF=0b
}
/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
