#define  USE_HY17M28_2M
/******************************************************************************
 * PWM_Demo.c                                                                 *
 * -------------------------------------------------------------------------- *
 * Copyright 2022 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 *   HY17M28                                                                  *
 * -----------                                                                *
 *           |                                                                *
 *     PT3.0 |-->PWM0                                                         *
 *           |                                                                *
 *     PT3.1 |-->PWM1                                                         *
 *           |                                                                *
 * -----------                                                                *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M28                                                           *
 * Project Name : PWM                                                         *
 * Created Date : 2022/3/1 By Cyril                                           *
 *                                                                            *
 ******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <TMR.h>
#include <CLK.h>
#include <INT.h>
#include <GPIO.h>
/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#ifdef USE_HY17M28_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M28_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M28_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M28_16M
#define HAO_16MHZ
#endif

////	Select PWM MODE		//
#define PWM1O
//#define PWM2O
//#define PWM3O
//#define PWM4O
//#define PWM5O
//#define PWM6O
//#define PWM7O

#define PWM1_Port  GPWM1_PT31
#define PWM0_Port  GPWM0_PT30
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void Delayms(unsigned int ms);
void CLOCKInit(void);
void PWMInit(void);

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
	CLOCKInit();
	PWMInit();


	while(1);

}
/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

}

/*----------------------------------------------------------------------------*/
/* PWM Init Function                                                        */
/*----------------------------------------------------------------------------*/
void PWMInit(void)
{
  //GPIO Init
  GPIO_GPWM0Set(PWM0_Port);    
  GPIO_GPWM1Set(PWM1_Port);    

//***** setting the timer B & PWM *********//
/*--------------------------------------PWM1O-------------------------------------------*/
#ifdef PWM1O
	TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB1M_16bit ,TB1RT_LogicH );

	TB1_PWM0_PHASE(PA0IV_NORMAL);		//PWM0 Output Phase Normal
	TB1_PWM1_PHASE(PA1IV_INVER);		//PWM1 Output Phase Normal

	TB1_PWM0ModeSelect(PWMA0_PWM1O);		//PWM0 Output mode Select PWM1O
	TB1_PWM1ModeSelect(PWMA1_PWM1O);		//PWM1 Output mode Select PWM1O

	TB1_PWMO0(PWMO0_OUTPUT);				//PWM0 Enable
	TB1_PWMO1(PWMO1_OUTPUT);				//PWM1 Enable

	TB1C0Set(0x0800);	// TB1C0[15:0] is setting the PWM frequency.
	TB1C1Set(0x0400);	// TB1C1[15:0] is setting the PWM Duty Cycle.
#endif

/*--------------------------------------PWM2O-------------------------------------------*/
#ifdef PWM2O
	TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB1M_16bit ,TB1RT_LogicH );

	TB1_PWM0_PHASE(PA0IV_NORMAL);		//PWM0 Output Phase Normal
	TB1_PWM1_PHASE(PA1IV_INVER);		//PWM1 Output Phase Normal

	TB1_PWM0ModeSelect(PWMA0_PWM2O);		//PWM0 Output mode Select PWM2O
	TB1_PWM1ModeSelect(PWMA1_PWM2O);		//PWM1 Output mode Select PWM2O

	TB1_PWMO0(PWMO0_OUTPUT);				//PWM0 Enable
	TB1_PWMO1(PWMO1_OUTPUT);				//PWM1 Enable

	TB1C0Set(0x0833);	// TB1C0[15:0] is setting the PWM frequency.
	TB1C2Set(0x01a4);	// TB1C2[15:0] is setting the PWM Duty Cycle.
#endif

/*--------------------------------------PWM3O-------------------------------------------*/
#ifdef PWM3O
	TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB1M_2_8bit ,TB1RT_LogicH );

	TB1_PWM0_PHASE(PA0IV_NORMAL);		//PWM0 Output Phase Normal
	TB1_PWM1_PHASE(PA1IV_INVER);		//PWM1 Output Phase Normal

	TB1_PWM0ModeSelect(PWMA0_PWM3O);		//PWM0 Output mode Select PWM3O
	TB1_PWM1ModeSelect(PWMA1_PWM3O);		//PWM1 Output mode Select PWM3O

	TB1_PWMO0(PWMO0_OUTPUT);				//PWM0 Enable
	TB1_PWMO1(PWMO1_OUTPUT);				//PWM1 Enable

	TB1C0Set(0x0029);	// TB1C0L[7:0] is setting the PWM frequency.
	TB1C1Set(0x0016);	// TB1C0L[7:0] is setting the PWM Duty Cycle.
#endif

/*--------------------------------------PWM4O-------------------------------------------*/
#ifdef PWM4O
	TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB1M_2_8bit ,TB1RT_LogicH );

	TB1_PWM0_PHASE(PA0IV_NORMAL);		//PWM0 Output Phase Normal
	TB1_PWM1_PHASE(PA1IV_INVER);		//PWM1 Output Phase Normal

	TB1_PWM0ModeSelect(PWMA0_PWM4O);		//PWM0 Output mode Select PWM4O
	TB1_PWM1ModeSelect(PWMA1_PWM4O);		//PWM1 Output mode Select PWM4O

	TB1_PWMO0(PWMO0_OUTPUT);				//PWM0 Enable
	TB1_PWMO1(PWMO1_OUTPUT);				//PWM1 Enable

	TB1C0Set(0xD100);	// TB1C0H[15:8] is setting the PWM frequency.
	TB1C2Set(0x0088);	// TB1C2L[7:0] is setting the PWM Duty Cycle.
#endif

/*--------------------------------------PWM5O-------------------------------------------*/
#ifdef PWM5O
	TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB1M_8_8bit ,TB1RT_LogicH );

	TB1_PWM0_PHASE(PA0IV_NORMAL);		//PWM0 Output Phase Normal
	TB1_PWM1_PHASE(PA1IV_INVER);		//PWM1 Output Phase Normal

	TB1_PWM0ModeSelect(PWMA0_PWM5O);		//PWM0 Output mode Select PWM5O
	TB1_PWM1ModeSelect(PWMA1_PWM5O);		//PWM1 Output mode Select PWM5O

	TB1_PWMO0(PWMO0_OUTPUT);				//PWM0 Enable
	TB1_PWMO1(PWMO1_OUTPUT);				//PWM1 Enable

	TB1C0Set(0x00C8);	// TB1C0L[7:0] is setting the PWM frequency.
	TB1C1Set(0x0064);	// TB1C1L[7:0] is setting the PWM Duty Cycle.
	TB1C2Set(0x0080);	// TB1C2L[7:0] is setting the PWM Duty Cycle fine tuning.
#endif

/*--------------------------------------PWM6O-------------------------------------------*/
#ifdef PWM6O
	TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB1M_17bit ,TB1RT_LogicH );

	TB1_PWM0_PHASE(PA0IV_NORMAL);		//PWM0 Output Phase Normal
	TB1_PWM1_PHASE(PA1IV_INVER);		//PWM1 Output Phase Normal

	TB1_PWM0ModeSelect(PWMA0_PWM6O);		//PWM0 Output mode Select PWM6O
	TB1_PWM1ModeSelect(PWMA1_PWM6O);		//PWM1 Output mode Select PWM6O

	TB1_PWMO0(PWMO0_OUTPUT);				//PWM0 Enable
	TB1_PWMO1(PWMO1_OUTPUT);				//PWM1 Enable

	TB1C0Set(0x0002);	// TB1C0L[7:0] is setting the PWM frequency.
	TB1C1Set(0x0001);	// TB1C1L[7:0] is setting the PWM Duty Cycle.
	TB1C2Set(0x0080);	// TB1C2L[7:0] is setting the PWM Duty Cycle
						// TB1C0 > TB1C2 > TB1C1
#endif

/*--------------------------------------PWM7O-------------------------------------------*/
#ifdef PWM7O
	TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV1 ,TB1M_16bit ,TB1RT_LogicH );

	TB1_PWM0_PHASE(PA0IV_NORMAL);		//PWM0 Output Phase Normal
	TB1_PWM1_PHASE(PA1IV_INVER);		//PWM1 Output Phase Normal

	TB1_PWM0ModeSelect(PWMA0_PWM7O);		//PWM0 Output mode Select PWM7O
	TB1_PWM1ModeSelect(PWMA1_PWM7O);		//PWM1 Output mode Select PWM7O

	TB1_PWMO0(PWMO0_OUTPUT);				//PWM0 Enable
	TB1_PWMO1(PWMO1_OUTPUT);				//PWM1 Enable

	TB1C0Set(0x3FFF);	// TB1C0[15:0] is setting the PWM frequency.
										// PWM Duty Cycle is always 50%
#endif
	TB1Enable();
}

/*----------------------------------------------------------------------------*/
/* Delay Function                                                             */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
	for(;num>0;num--)
    __asm__("NOP");
}


void Delayms(unsigned int ms)
{
  unsigned char t;
  for(;ms>0;ms--)        //@HAO=1.843MHz
    for(t=114;t>0;t--)
    { __asm__("NOP"); }
}

/*----------------------------------------------------------------------------*/
/* End Of File                                       */
/*----------------------------------------------------------------------------*/

