#define  USE_HY17M28_2M
/******************************************************************************
 * -------------------------------------------------------------------------- *
 * Copyright 2022 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M28                                                           *
 *****************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <INT.h>
#include <CLK.h>
#include <RST.h>
#include <UART.h>
#include <PWR.h>
#include <ADC.h>
#include <TMR.h>
#include <GPIO.h>
#include <LED.h>
#include <TP.h>
#include "lcdTable.h"
/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
//#define ADC_ChopperMode

#ifdef USE_HY17M28_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M28_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M28_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M28_16M
#define HAO_16MHZ
#endif

#define UART_Port GTX_PT13

#define  Uart_RX_BufferSize 10
#define  Uart_TX_BufferSize 10
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int del);
void CLOCKInit(void);
void LED_Display_Init(void);
void LED_Display_show(signed long DATA);
void LED_Display_h(unsigned char ledx, unsigned char On_OFF);
void ADCInit(void);
void UARTInit(void);
void SendUART(signed long  data);
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
unsigned char LED[8];
unsigned char LEDScan;

volatile typedef union _Flag
{
 unsigned int  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_UART_TxDone:1;
    unsigned b_UART_RxDone:1;
    unsigned Reserved:13;
  };
} Flag;

//signed long
//range 8388607~-8388608 (dec)
//range 0x007F FFFF ~ 0xFF80 0000 (hex)
signed long  ADC_ISRTemp;
signed long  ADCData1;
Flag   Flagbits;
unsigned char UartTxBuffer[Uart_TX_BufferSize]={0};
unsigned char UartRxBuffer[Uart_RX_BufferSize]={0};
unsigned char UartTxIndex,UartTxLength;
unsigned char UartRxIndex,UartRxLength;


signed long ADC_NO_TOUCH_TK1=-200000;
signed long ADC_TK1=0;
signed long ADC_TK1_Thresould=-300000;
unsigned int ADC_TK1_COUNT=0;
/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{

  //Init data
  UartTxIndex=0;
  UartRxIndex=0;
  Flagbits.b_ADCdone=0;
  Flagbits.b_UART_TxDone=0;
  Flagbits.b_UART_RxDone=0;

  //PT20 LED setting
  LED_Display_Init();
  
  //CLOCK+ADC+UART Init
  CLOCKInit();
  ADCInit();
  UARTInit();
  Flagbits.b_UART_TxDone=1; //Avoid UART Enable error on first data

  GIE_Enable();

  while(1)
  {
  	Flagbits.b_ADCdone=0;
    while(Flagbits.b_ADCdone==0);  //until get ADC_ISRTemp
    Flagbits.b_ADCdone=0;

    ADC_INT_Disable();
    ADC_TK1=ADC_ISRTemp;
    if((ADC_TK1-ADC_NO_TOUCH_TK1)<ADC_TK1_Thresould)
    {
      ADC_TK1_COUNT++;
    }
    LED_Display_show(ADC_TK1_COUNT);
    Delay(10000);
    SendUART(ADC_TK1);
    ADC_INT_Enable();
  }

}
/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt
{
    //TMA Event
	if(TA1CIF_IsFlag())
	{
		TA1CIF_ClearFlag();
		TMA1R=0;
		LEDScan=LEDScan>>1;
		if(LEDScan==0)
		{LEDScan=0x80;}

		switch(LEDScan)
		{
			case 0x01:
			{
				PT2= 0;
				TRISC2=LED[0] | 0x01;
				PT2=~(LED[0] & 0xFE);
				break;
			}
			case 0x02:
			{
				PT2= 0;
				TRISC2=LED[1] | 0x02;
				PT2=~(LED[1]& 0xFD);
				break;
			}
			case 0x04:
			{
				PT2= 0;
				TRISC2=LED[2] | 0x04;
				PT2=~(LED[2] & 0xFB);
				break;
			}
			case 0x08:
			{
				PT2= 0;
				TRISC2=LED[3] | 0x08;
				PT2=~(LED[3]& 0xF7);
				break;
			}
			case 0x10:
			{
				PT2= 0;
				TRISC2=LED[4] | 0x10;
				PT2=~(LED[4]& 0xEF);
				break;
			}
			case 0x20:
			{
				PT2= 0;
				TRISC2=LED[5] | 0x20;
				PT2=~(LED[5]& 0xDF);
				break;
			}
			case 0x40:
			{
				PT2= 0;
				TRISC2=LED[6] | 0x40;
				PT2=~(LED[6]& 0xBF);
				break;
			}
			case 0x80:
			{
				PT2= 0;
				TRISC2=LED[7] | 0x80;
				PT2=~(LED[7]& 0x7F);
				break;
			}
		}
	}

  //ADC Event
  if(ADC_INT_IsFlag())
  {
    ADC_INT_ClearFlag();
    ADC_ISRTemp=ADCR; //ADC_GetData();
    Flagbits.b_ADCdone=1;
  }

  //UART0 RX Event
  if(UART0_INT_RCIsFlag())
  {
    UartRxBuffer[UartRxIndex]=RC0REG;
    UartRxIndex++;
    if(UartRxIndex>=Uart_RX_BufferSize)
    {
      UartRxIndex=0;
      Flagbits.b_UART_RxDone=1;
    }
    UART0_INT_RCClearFlag();
  }

  //UART0 TX Event
  if(UART0_INT_TXIsFlag())
  {
    if(Flagbits.b_UART_TxDone==0)
    {
      TX0R=UartTxBuffer[UartTxIndex++];
      UART0_INT_TXClearFlag();
      if(UartTxIndex>=UartTxLength)
      {
        UART0_INT_TXEnable();    //TXIE=1b
        UART0_INT_RCEnable();  //RCIE=1b
        Flagbits.b_UART_TxDone=1;
        UartTxIndex=0;
      }
    }
    if(Flagbits.b_UART_TxDone==1)
    {
      UART0_INT_TXDisable();    //TXIE=0b
      UART0_INT_RCEnable();  //RCIE=1b
      UART0_INT_TXClearFlag();
    }
  }


}
/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

}
/*----------------------------------------------------------------------------*/
/* ADC Init Function                                                          */
/*----------------------------------------------------------------------------*/
void ADCInit(void)
{
  //==========Setup Power=====================================
  PWR_BGREnable();			//bandgap Vref Enable
  PWR_VDDAOpen(LDOC_2V4);	//VDDA Enable, refer to the Datasheet for LDOC typical voltage

  //==========Setup ADC =================================
#if defined(HAO_2MHZ)
  //HAO=2MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV2,INP_TKP,INN_TKN,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_16384);
#endif

#if defined(HAO_4MHZ)
  //HAO=4MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV4,INP_TKP,INN_TKN,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_16384);
#endif

#if defined(HAO_8MHZ)
  //HAO=8MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV8,INP_TKP,INN_TKN,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_16384);
#endif

#if defined(HAO_16MHZ)
  //HAO=15.667MHz, ADC_CK=1MHz
  ADC_Open(DADC_DHSCKDIV16,INP_TKP,INN_TKN,VRH_VDDA,VRL_VSS,ADGN_1,VREGN_DIV2,DCSET_P0,OSR_16384);
#endif


  //==========Setup TK1 =======================================
  GPIO_GTKTXSet(GTKTX_PT10);  //GTKTX=01b, TX1=PT10
  TP_CfbSelect(RxCfb_10p);    //Φ]wRx_Cfb=10p, SĲIb-217599 (TX1/RX1)
  //TP_CfbSelect(RxCfb_40p);  //Φ]wRx_Cfb=40p, SĲIb-38608 (TX1/RX1)
  TP_CKDivSelect(TPCLK_HAODIV2); //TP_CK= HAO/2
  TP_MeasureChSet(MRx_Single);  //Touch Key 1 Channel
  TP_MeasureOrder(RxMx_RX1);  //RxMx=RX1(PT35)
  TP_StabilizeTime(TxSettle_3Clock);  //TXSettle=3*TX Clock time
  TP_CommonVoltageSel(TKCMS_TKBuffer);  //TKCMS= Touch key common mode buffer
  GPIO_PT3SETDA(5);
  TP_Enable();  //Enable touch key


#ifdef ADC_ChopperMode
  CSFON_Enable();
  ADC_ENINXCH_Enable();
  ADC_DAFM_CHOPPER();
  ADC_ENCH_Enable();
#endif
  ADC_Enable();
  ADC_CMFREnable();     //CMFR=1, Comb Filter Reset
}

/*----------------------------------------------------------------------------*/
/* LED_Display_show Function                                                      */
/*----------------------------------------------------------------------------*/
void LED_Display_show(signed long DATA)
{
  //display range +8388607~-8388608 (dec)
  //display range 0x7FFFFF~ 0x800000 (hex)
  ADCData1=DATA;
  if((ADCData1<0)||(ADCData1>0x80000000))  // plus-minus sign judgment
  {
	  ADCData1=~ADCData1;
	  ADCData1++;
      LED[7]=LED[7] | 0x40;  //UartTxBuffer[0]='-';
  }
  else
  {
      if(LED[7]==0x40)
      {LED[7]=LED[7] - 0x40;}  //UartTxBuffer[0]='+';
  }
  DATA=ADCData1;

  ADCData1= seg[DATA%10];
  LED[6]= GetDisplay(ADCData1,7);
  DATA /=10;

  ADCData1= seg[DATA%10];
  LED[5]= GetDisplay(ADCData1,6);
  DATA /=10;

  ADCData1= seg[DATA%10];
  LED[4]= GetDisplay(ADCData1,5);
  DATA /=10;

  ADCData1= seg[DATA%10];
  LED[3]= GetDisplay(ADCData1,4);
  DATA /=10;

  ADCData1= seg[DATA%10];
  LED[2]= GetDisplay(ADCData1,3);
  DATA /=10;

  ADCData1= seg[DATA%10];
  LED[1]= GetDisplay(ADCData1,2);
  DATA /=10;

  ADCData1= seg[DATA%10];
  LED[0]= GetDisplay(ADCData1,1);

}

/*----------------------------------------------------------------------------*/
/* LED Init Function                                                          */
/*----------------------------------------------------------------------------*/
void LED_Display_Init(void)
{
  LEDSetting(CCLevel_15mA);     //Set PT2 constant current 15mA
  LEDScan= 0;

  TMA1_Open(TMAS_DMSCK,DTMA_TMACKDIV2,2);   //Enable TMA, set LED scan rate=2
  TA1IE_Disable();    //Disable TMA1 Interrupt
  TA1CIE_Enable();    //Enable TMA1C Interrupt

}
/*----------------------------------------------------------------------------*/
/* LED_Display_h Function                                                         */
/*----------------------------------------------------------------------------*/
void LED_Display_h(unsigned char ledx, unsigned char On_OFF)
{

  unsigned char LEDScan_h, On_OFF_h ;
  LEDScan_h=ledx;
  On_OFF_h=On_OFF;

  switch(LEDScan_h)
  {
    case 0x1:
    {
      if(On_OFF_h==1)
      {LED[7]=LED[7] | 0x01;}
      if(On_OFF_h==0)
      {LED[7]=LED[7] -0x01;}
      break;
    }
    case 0x2:
    {
      if(On_OFF_h==1)
      {LED[7]=LED[7] | 0x02;}
      if(On_OFF_h==0)
      {LED[7]=LED[7] - 0x02;}
      break;
    }
    case 0x3:
    {
      if(On_OFF_h==1)
      {LED[7]=LED[7] | 0x04;}
      if(On_OFF_h==0)
      {LED[7]=LED[7] - 0x04;}
      break;
    }
    case 0x4:
    {
      if(On_OFF_h==1)
      {LED[7]=LED[7] | 0x08;}
      if(On_OFF_h==0)
      {LED[7]=LED[7] - 0x08;}
      break;
    }
    case 0x5:
    {
      if(On_OFF_h==1)
      {LED[7]=LED[7] | 0x10;}
      if(On_OFF_h==0)
      {LED[7]=LED[7] - 0x10;}
      break;
    }
    case 0x6:
    {
      if(On_OFF_h==1)
      {LED[7]=LED[7] | 0x20;}
      if(On_OFF_h==0)
      {LED[7]=LED[7] - 0x20;}
      break;
    }
    case 0x7:
    {
      if(On_OFF_h==1)
      {LED[7]=LED[7] | 0x40;}
      if(On_OFF_h==0)
      {LED[7]=LED[7] - 0x40;}
      break;
    }
  }

}

/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int del)
{
  while(--del)
  NOP();
}

/*----------------------------------------------------------------------------*/
/* UART Init Function                                                          */
/*----------------------------------------------------------------------------*/
void UARTInit(void)
{
  //GPIO Init
  GPIO_GTXSet(UART_Port);  //TX PT1.3 Rx PT1.4

#ifndef UART_ABD
    //BRGR[15:0]=((CPUCK/Baudrate)/4)-1
  #if defined(HAO_2MHZ)
    //((1843000/9600)/4)-1 =47, HAO=1.843M, baudrate=9600
    UART0_Open(47 ,8 ,PARITY_None);
  #endif
  #if defined(HAO_4MHZ)
    //((4147000/9600)/4)-1 =107, HAO=4.147M, baudrate=9600
    UART0_Open(107 ,8 ,PARITY_None);
  #endif
  #if defined(HAO_8MHZ)
    //((8755000/9600)/4)-1 =227, HAO=8.755M, baudrate=9600
    UART0_Open(227 ,8 ,PARITY_None);
  #endif
  #if defined(HAO_16MHZ)
    //((15667000/9600)/4)-1 =407, HAO=15.667M, baudrate=9600
    UART0_Open(407 ,8 ,PARITY_None);
  #endif
#else
  UART0_Open(0 ,8 ,PARITY_None);
  UART0_WUEDisable();                   // Wake-up disable
  UART0_ABDEnable();                    //Enable Auto Baudrate
  while((BA0CN & 0x01) == 1 );      //Wait for master send 055H
  UART0_INT_RCClearFlag();          //Clear RC interrupt flag

#endif
  TXIE_Enable();
  RCIE_Enable();
}

/*----------------------------------------------------------------------------*/
/* Function Name: void SendUART(int data)                                     */
/* Description  : UART send data.                                             */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void SendUART(signed long  data)
{
  //display range +8388607~-8388608 (dec)
  ADCData1=data;
  if((ADCData1<0)||(ADCData1>0x80000000))  // plus-minus sign judgment
  {
	  ADCData1=~ADCData1;
	  ADCData1++;
      UartTxBuffer[0]='-';
  }
  else
  {
    UartTxBuffer[0]='+';
  }
  UartTxBuffer[7]=ADCData1%10 | '0';  //unit
  ADCData1=ADCData1/10;
  UartTxBuffer[6]=ADCData1%10 | '0';  //unit
  ADCData1=ADCData1/10;
  UartTxBuffer[5]=ADCData1%10 | '0';  //ten
  ADCData1=ADCData1/10;
  UartTxBuffer[4]=ADCData1%10 | '0';  //hundred
  ADCData1=ADCData1/10;
  UartTxBuffer[3]=ADCData1%10 | '0';  //thousand
  ADCData1=ADCData1/10;
  UartTxBuffer[2]=ADCData1%10 | '0';  //ten thousand
  ADCData1=ADCData1/10;
  UartTxBuffer[1]=ADCData1%10 | '0';  //hundred thousand
  ADCData1=ADCData1/10;
  UartTxBuffer[8]='\r';
  UartTxBuffer[9]='\n';
  Flagbits.b_UART_TxDone=0;
  UartTxLength=10;
  UartTxIndex=0;
  TXIE_Enable();    //Enable UART Tx Interrupt;
  RCIE_Enable();   //Enable UART Rx Interrupt
  while(!Flagbits.b_UART_TxDone);  //If  Flagbits.b_UART_TxDone=DISABLE, stop at here
}

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
