#define  USE_HY17M28_2M
/******************************************************************************
 * TMB_Demo.c                                                                 *
 * -------------------------------------------------------------------------- *
 * Copyright 2022 HYCON Technology                                            *
 * http://www.hycontek.com/                                                   *
 *                                                                            *
 * Program Description:                                                       *
 *                                                                            *
 *        HY17M28                                                             *
 *--------------------                                                        *
 *                    |                                                       *
 *               PT2.0|->_|-|_ TMB                                            *
 *                    |                                                       *
 *--------------------                                                        *
 *                                                                            *
 * For External Input                                                         *
 * IC Body: HY17M28                                                           *
 * Project Name : TMB                                                         *
 * Created Date : 2022/3/1 BY Cyril                                           *
 *                                                                            *
 ******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include <SFRType.h>
#include <TMR.h>
#include <CLK.h>
#include <INT.h>
#include <GPIO.h>

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/
#ifdef USE_HY17M28_2M
#define HAO_2MHZ
#endif
#ifdef USE_HY17M28_4M
#define HAO_4MHZ
#endif
#ifdef USE_HY17M28_8M
#define HAO_8MHZ
#endif
#ifdef USE_HY17M28_16M
#define HAO_16MHZ
#endif

#define mode_16bit
//#define mode_17bit
//#define mode_2_8bit
//#define mode_8_8bit

/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num);
void Delayms(unsigned int ms);
void CLOCKInit(void);
void TMBInit(void);
void GPIOInit(void);
/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
void main(void)
{
	CLOCKInit();
	TMBInit();
	GPIOInit();

	GIE_Enable();
	while(1);
}

/*----------------------------------------------------------------------------*/
/* Interrupt Service Routines                                                 */
/*----------------------------------------------------------------------------*/
void ISR(void) __interrupt
{
	if(TB1IF_IsFlag())		//TB1R=TB1C ,TMBIF=1
	{
		PT3= PT3^PT30_MSK;  //PT3.0 Inverted
		TB1IF_ClearFlag();
	}
}

/*----------------------------------------------------------------------------*/
/* Subroutine Function                                                        */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/* Delay Function                                                             */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
	for(;num>0;num--)
    __asm__("NOP");
}


void Delayms(unsigned int ms)
{
  unsigned char t;
  for(;ms>0;ms--)        //@HAO=1.843MHz
    for(t=114;t>0;t--)
    { __asm__("NOP"); }
}

/*----------------------------------------------------------------------------*/
/* TMB Init Function                                                        */
/*----------------------------------------------------------------------------*/
void TMBInit(void)		//select TMB mode
{
#if defined(mode_16bit)
  TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV2 ,TB1M_16bit ,TB1RT_LogicH );
  TB1C0Set(0x00ff);	//Set TMB count
#endif
#if defined(mode_17bit)
  TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV2 ,TB1M_17bit ,TB1RT_LogicH );
  TB1C0Set(0x02ff);	//Set TMB count
#endif
#if defined(mode_2_8bit)
  TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV2 ,TB1M_2_8bit ,TB1RT_LogicH );
  TB1C0Set(0x7fff);	//Set TMB count
#endif
#if defined(mode_8_8bit)
  TMB1_Open(TMBS_HSCK ,DTMB_TMBCKDIV2 ,TB1M_8_8bit ,TB1RT_LogicH );
  TB1C0Set(0x7fff);	//Set TMB count
#endif
  TB1Enable();
}

/*----------------------------------------------------------------------------*/
/* CLOCK Init Function                                                        */
/*----------------------------------------------------------------------------*/
void CLOCKInit(void)
{//Please refer to the Datasheet for HAOM center frequency
#if defined(HAO_2MHZ)
  CLK_CPUCKOpen(HAOM_2MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_4MHZ)
  CLK_CPUCKOpen(HAOM_4MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_8MHZ)
  CLK_CPUCKOpen(HAOM_8MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

#if defined(HAO_16MHZ)
  CLK_CPUCKOpen(HAOM_16MHZ ,OSCS_HAO ,DHS_HSCKDIV1 ,CPUS_HSCK );
#endif

}

/*----------------------------------------------------------------------------*/
/* GPIO Init Function                                                        */
/*----------------------------------------------------------------------------*/
void GPIOInit(void)
{
  GPIO_PT3OutputMode(PT30);
  GPIO_PT3DigitalEnable(PT30);
  GPIO_PT3OutputLow(PT30);

}

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
